@@ -8,52 +8,15 @@ details of the supported formats, conventions and conversions.
 
 """
 
-# Stylistic notes:
-#
-# The HTML5 algorithms are implemented as direct translations into
-# Python of the descriptions in the spec. This produces somewhat
-# un-Pythonic code, but correctness of the implementations is more
-# important.
-#
-# Several other style choices in this module enforce usage or Python
-# version support (see documentation in docs/ directory for rationale
-# behind Python version support and usage requirements):
-#
-# * The only Python 2.x version supported is 2.7. Use of a dict
-#   comprehension in the _reversedict() helper function enforces this
-#   by causing an import-time syntax error on Python 2.6 and earlier.
-#
-# * The oldest Python 3.x version supported is 3.3. Use of u-prefixed
-#   string literals enforces this by causing an import-time syntax
-#   error on Python 3.0, 3.1 and 3.2.
-#
-# * Use of this module on Python 3 requires str rather than bytes for
-#   all string arguments to functions. Use of the format() method for
-#   string formatting enforces this; as of Python 3.5, formatting with
-#   % is implemented on bytes objects, but the format() method will
-#   never be implemented on bytes.
-
-
 import collections
 import re
 import string
 import struct
 
-
-__version__ = '1.8'
+import six
 
 
-# Python 2's unichr() is Python 3's chr().
-try:               # pragma: no cover
-    unichr         # pragma: no cover
-except NameError:  # pragma: no cover
-    unichr = chr   # pragma: no cover
-
-# Python 2's unicode is Python 3's str.
-try:               # pragma: no cover
-    unicode        # pragma: no cover
-except NameError:  # pragma: no cover
-    unicode = str  # pragma: no cover
+__version__ = '1.9.1'
 
 
 def _reversedict(d):
@@ -67,7 +30,12 @@ def _reversedict(d):
 
 HEX_COLOR_RE = re.compile(r'^#([a-fA-F0-9]{3}|[a-fA-F0-9]{6})$')
 
-SUPPORTED_SPECIFICATIONS = (u'html4', u'css2', u'css21', u'css3')
+HTML4 = u'html4'
+CSS2 = u'css2'
+CSS21 = u'css21'
+CSS3 = u'css3'
+
+SUPPORTED_SPECIFICATIONS = (HTML4, CSS2, CSS21, CSS3)
 
 SPECIFICATION_ERROR_TEMPLATE = u'{{spec}} is not a supported specification for color name lookups; \
 supported specifications are: {supported}.'.format(
@@ -303,6 +271,18 @@ CSS21_HEX_TO_NAMES = _reversedict(CSS21_NAMES_TO_HEX)
 
 CSS3_HEX_TO_NAMES = _reversedict(CSS3_NAMES_TO_HEX)
 
+# Both 'gray' and 'grey' are accepted in CSS3, and both map to the
+# same 24-bit value (#808080). Reversing this requires picking one
+# spelling to be the "winner".
+#
+# The way in which _reversedict generates these mappings will pick a
+# "winner" based on the ordering of dictionary keys, which varies
+# according to the Python version in use, and on some Python versions
+# is deliberately not to be relied on for consistency. So we manually
+# pick one. Since it was the only spelling supported by HTML 4, CSS1,
+# and CSS2, 'gray' is the winner.
+CSS3_HEX_TO_NAMES[u'#808080'] = u'gray'
+
 
 # Aliases of the above mappings, for backwards compatibility.
 #################################################################
@@ -392,14 +372,13 @@ def normalize_percent_triplet(rgb_triplet):
 # Conversions from color names to various formats.
 #################################################################
 
-def name_to_hex(name, spec=u'css3'):
+def name_to_hex(name, spec=CSS3):
     """
     Convert a color name to a normalized hexadecimal color value.
 
     The optional keyword argument ``spec`` determines which
-    specification's list of color names will be used; valid values are
-    ``html4``, ``css2``, ``css21`` and ``css3``, and the default is
-    ``css3``.
+    specification's list of color names will be used. The default is
+    CSS3.
 
     When no color of that name exists in the given specification,
     ``ValueError`` is raised.
@@ -408,10 +387,12 @@ def name_to_hex(name, spec=u'css3'):
     if spec not in SUPPORTED_SPECIFICATIONS:
         raise ValueError(SPECIFICATION_ERROR_TEMPLATE.format(spec=spec))
     normalized = name.lower()
-    hex_value = {u'css2': CSS2_NAMES_TO_HEX,
-                 u'css21': CSS21_NAMES_TO_HEX,
-                 u'css3': CSS3_NAMES_TO_HEX,
-                 u'html4': HTML4_NAMES_TO_HEX}[spec].get(normalized)
+    hex_value = {
+        CSS2: CSS2_NAMES_TO_HEX,
+        CSS21: CSS21_NAMES_TO_HEX,
+        CSS3: CSS3_NAMES_TO_HEX,
+        HTML4: HTML4_NAMES_TO_HEX
+    }[spec].get(normalized)
     if hex_value is None:
         raise ValueError(
             u"'{name}' is not defined as a named color in {spec}".format(
@@ -421,7 +402,7 @@ def name_to_hex(name, spec=u'css3'):
     return hex_value
 
 
-def name_to_rgb(name, spec=u'css3'):
+def name_to_rgb(name, spec=CSS3):
     """
     Convert a color name to a 3-tuple of integers suitable for use in
     an ``rgb()`` triplet specifying that color.
@@ -430,7 +411,7 @@ def name_to_rgb(name, spec=u'css3'):
     return hex_to_rgb(name_to_hex(name, spec=spec))
 
 
-def name_to_rgb_percent(name, spec=u'css3'):
+def name_to_rgb_percent(name, spec=CSS3):
     """
     Convert a color name to a 3-tuple of percentages suitable for use
     in an ``rgb()`` triplet specifying that color.
@@ -442,15 +423,14 @@ def name_to_rgb_percent(name, spec=u'css3'):
 # Conversions from hexadecimal color values to various formats.
 #################################################################
 
-def hex_to_name(hex_value, spec=u'css3'):
+def hex_to_name(hex_value, spec=CSS3):
     """
     Convert a hexadecimal color value to its corresponding normalized
     color name, if any such name exists.
 
     The optional keyword argument ``spec`` determines which
-    specification's list of color names will be used; valid values are
-    ``html4``, ``css2``, ``css21`` and ``css3``, and the default is
-    ``css3``.
+    specification's list of color names will be used. The default is
+    CSS3.
 
     When no color name for the value is found in the given
     specification, ``ValueError`` is raised.
@@ -459,10 +439,12 @@ def hex_to_name(hex_value, spec=u'css3'):
     if spec not in SUPPORTED_SPECIFICATIONS:
         raise ValueError(SPECIFICATION_ERROR_TEMPLATE.format(spec=spec))
     normalized = normalize_hex(hex_value)
-    name = {u'css2': CSS2_HEX_TO_NAMES,
-            u'css21': CSS21_HEX_TO_NAMES,
-            u'css3': CSS3_HEX_TO_NAMES,
-            u'html4': HTML4_HEX_TO_NAMES}[spec].get(normalized)
+    name = {
+        CSS2: CSS2_HEX_TO_NAMES,
+        CSS21: CSS21_HEX_TO_NAMES,
+        CSS3: CSS3_HEX_TO_NAMES,
+        HTML4: HTML4_HEX_TO_NAMES
+    }[spec].get(normalized)
     if name is None:
         raise ValueError(
             u"'{}' has no defined color name in {}".format(hex_value, spec)
@@ -497,16 +479,15 @@ def hex_to_rgb_percent(hex_value):
 # Conversions from  integer rgb() triplets to various formats.
 #################################################################
 
-def rgb_to_name(rgb_triplet, spec=u'css3'):
+def rgb_to_name(rgb_triplet, spec=CSS3):
     """
     Convert a 3-tuple of integers, suitable for use in an ``rgb()``
     color triplet, to its corresponding normalized color name, if any
     such name exists.
 
     The optional keyword argument ``spec`` determines which
-    specification's list of color names will be used; valid values are
-    ``html4``, ``css2``, ``css21`` and ``css3``, and the default is
-    ``css3``.
+    specification's list of color names will be used. The default is
+    CSS3.
 
     If there is no matching name, ``ValueError`` is raised.
 
@@ -562,16 +543,15 @@ def rgb_to_rgb_percent(rgb_triplet):
 # Conversions from percentage rgb() triplets to various formats.
 #################################################################
 
-def rgb_percent_to_name(rgb_percent_triplet, spec=u'css3'):
+def rgb_percent_to_name(rgb_percent_triplet, spec=CSS3):
     """
     Convert a 3-tuple of percentages, suitable for use in an ``rgb()``
     color triplet, to its corresponding normalized color name, if any
     such name exists.
 
     The optional keyword argument ``spec`` determines which
-    specification's list of color names will be used; valid values are
-    ``html4``, ``css2``, ``css21`` and ``css3``, and the default is
-    ``css3``.
+    specification's list of color names will be used. The default is
+    CSS3.
 
     If there is no matching name, ``ValueError`` is raised.
 
@@ -658,7 +638,7 @@ def html5_parse_simple_color(input):
     #
     # 2. If input is not exactly seven characters long, then return an
     #    error.
-    if not isinstance(input, unicode) or len(input) != 7:
+    if not isinstance(input, six.text_type) or len(input) != 7:
         raise ValueError(
             u"An HTML5 simple color must be a Unicode string "
             u"exactly seven characters long."
@@ -730,7 +710,7 @@ def html5_parse_legacy_color(input):
 
     """
     # 1. Let input be the string being parsed.
-    if not isinstance(input, unicode):
+    if not isinstance(input, six.text_type):
         raise ValueError(
             u"HTML5 legacy color parsing requires a Unicode string as input."
         )
@@ -825,7 +805,7 @@ def html5_parse_legacy_color(input):
     format_string = '<' + ('L' * (int(len(encoded_input) / 4)))
     codepoints = struct.unpack(format_string, encoded_input)
     input = ''.join(u'00' if c > 0xffff
-                    else unichr(c)
+                    else six.unichr(c)
                     for c in codepoints)
 
     # 8. If input is longer than 128 characters, truncate input,
